@@ -1,9 +1,7 @@
-#ifndef READXL_ZIP_
-#define READXL_ZIP_
+#pragma once
 
-#include "rapidxml.h"
+#include "rapidxml/rapidxml.h"
+#include <string>
 
 std::string zip_buffer(const std::string& zip_path, const std::string& file_path);
 bool zip_has_file(const std::string& zip_path, const std::string& file_path);
-
-#endif
